# Copyright 2018 Globo.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" Tests for faasclient.client methods """

import unittest
from mock import patch
from faasclient.client import Client


class TestFaasclient(unittest.TestCase):

    def setUp(self):
        self.client = Client()

    # REQUESTS
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._request')
    def test_get_request(self, mock_request):

        url_path = '/some/path'

        self.client._get_request(url_path)
        mock_request.assert_called_with('get', url_path, headers=None)

        header = {'header': 'value'}

        self.client._get_request(url_path, headers=header)
        mock_request.assert_called_with('get', url_path, headers=header)

    @patch('faasclient.client.Client._request')
    def test_post_request(self, mock_request):

        url_path = '/some/path'
        data = {'name': 'any name'}

        self.client._post_request(url_path, data)
        mock_request.assert_called_with('post', url_path, data=data,
                                        headers=None, audit=False)

        header = {'header': 'value'}

        self.client._post_request(url_path, data, headers=header)
        mock_request.assert_called_with('post', url_path, data=data,
                                        headers=header, audit=False)

    @patch('faasclient.client.Client._request')
    def test_put_request(self, mock_request):

        url_path = '/some/path'
        data = {'name': 'any name'}

        self.client._put_request(url_path, data)
        mock_request.assert_called_with('put', url_path, data=data,
                                        headers=None, audit=False)

        header = {'header': 'value'}

        self.client._put_request(url_path, data, headers=header)
        mock_request.assert_called_with('put', url_path, data=data,
                                        headers=header, audit=False)

    @patch('faasclient.client.Client._request')
    def test_delete_request(self, mock_request):

        url_path = '/some/path'

        self.client._delete_request(url_path)
        mock_request.assert_called_with('delete', url_path, audit=False, headers=None)

        header = {'header': 'value'}

        self.client._delete_request(url_path, headers=header)
        mock_request.assert_called_with('delete', url_path, audit=False, headers=header)

    # CATEGORIA
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._get_request')
    def test_categoria_list(self, mock_request):

        self.client.categoria_list()
        mock_request.assert_called_once_with('/categorias')

    @patch('faasclient.client.Client._post_request')
    def test_categoria_create(self, mock_request):

        name = 'categoria'

        self.client.categoria_create(name)
        mock_request.assert_called_once_with('/categorias', {'name': name})

    @patch('faasclient.client.Client._put_request')
    def test_categoria_update(self, mock_request):

        id = 123
        name = 'nova categoria'

        self.client.categoria_update(id, name)
        mock_request.assert_called_once_with('/categorias/%d' % id, {
                                             'name': name
                                             })

    @patch('faasclient.client.Client._delete_request')
    def test_categoria_delete(self, mock_request):

        id = 123

        self.client.categoria_delete(id)
        mock_request.assert_called_once_with('/categorias/%d' % id)

    # FILER
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._get_request')
    def test_export_get(self, mock_request):

        id = 123

        self.client.export_get(id)
        mock_request.assert_called_once_with('/exports/%d' % id)

    @patch('faasclient.client.Client._get_request')
    def test_export_list(self, mock_request):

        self.client.export_list()
        mock_request.assert_called_once_with('/exports')

    @patch('faasclient.client.Client._get_request')
    def test_export_list_with_host_parameter(self, mock_request):

        self.client.export_list(host='9.9.9.9')
        mock_request.assert_called_once_with('/exports?host=9.9.9.9')

    @patch('faasclient.client.Client._get_request')
    def test_export_list_with_extra_parameter(self, mock_request):

        self.client.export_list(extra='blah')
        mock_request.assert_called_once_with('/exports?extra=blah')

    @patch('faasclient.client.Client._get_request')
    def test_export_list_with_host_and_extra_parameters(self, mock_request):

        self.client.export_list(host='9.9.9.9', extra='blah')
        mock_request.assert_called_once_with('/exports?host=9.9.9.9&extra=blah')

    @patch('faasclient.client.Client._post_request')
    def test_export_create(self, mock_request):

        quota = 123456789
        categoria_id = 1
        time_id = '123456'

        self.client.export_create(quota, categoria_id, time_id)
        mock_request.assert_called_once_with('/exports', {
            'quota': quota, 'categoria': categoria_id, 'resource_id': None, 'time_id': '123456'}, audit=False)

    @patch('faasclient.client.Client._post_request')
    def test_export_create_with_resource(self, mock_request):

        quota = 123456789
        categoria_id = 1
        time_id = '123456'
        resource_id = 'aaaa-bbbb-cccc'

        self.client.export_create(quota, categoria_id, time_id, resource_id)
        mock_request.assert_called_once_with('/exports', {
            'quota': quota, 'categoria': categoria_id, 'time_id': '123456', 'resource_id': 'aaaa-bbbb-cccc'}, audit=False)

    @patch('faasclient.client.Client._post_request')
    def test_export_create_multiple_with_amount(self, mock_request):

        quota = 123456789
        categoria_id = 1
        time_id = '123456'
        resource_id = 'aaaa-bbbb-cccc'
        amount = 3

        self.client.export_multiple_create(quota, categoria_id, time_id, resource_id=resource_id, amount=amount)
        mock_request.assert_called_once_with('/exports/multiple', {
            'quota': quota, 'categoria': categoria_id, 'time_id': '123456', 'resource_id': 'aaaa-bbbb-cccc', 'amount': 3}, audit=False)

    @patch('faasclient.client.Client._delete_request')
    def test_export_delete(self, mock_request):

        id = 123

        self.client.export_delete(id)
        mock_request.assert_called_once_with('/exports/%d' % id, audit=False)

    @patch('faasclient.client.Client._delete_request')
    def test_export_force_delete(self, mock_request):

        id = 123

        self.client.export_force_delete(id)
        mock_request.assert_called_once_with('/exports/%d/force' % id, audit=False)

    # FILER
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._get_request')
    def test_filer_list(self, mock_request):

        self.client.filer_list()
        mock_request.assert_called_once_with('/filers')

    @patch('faasclient.client.Client._post_request')
    def test_filer_create(self, mock_request):

        vfiler = 'vbla'
        export_filer = 'bla'
        connection_filer = 'blabla'
        username = 'fake'
        password = 12345
        vserver = 'vbla'

        self.client.filer_create(vfiler, export_filer, connection_filer,
                                 username, password, vserver)
        mock_request.assert_called_with('/filers',
                                        {'vfiler': vfiler,
                                         'connection_filer': connection_filer,
                                         'export_filer': export_filer,
                                         'username': username,
                                         'password': password,
                                         'vserver': vserver
                                         }
                                        )

    @patch('faasclient.client.Client._get_request')
    def test_filer_get(self, mock_request):

        id = 123

        self.client.filer_get(id)
        mock_request.assert_called_once_with('/filers/%d' % id)

    @patch('faasclient.client.Client._delete_request')
    def test_filer_delete(self, mock_request):

        id = 123

        self.client.filer_delete(id)
        mock_request.assert_called_once_with('/filers/%d' % id)

    @patch('faasclient.client.Client._put_request')
    def test_filer_update(self, mock_request):

        fakeid = 123
        vfiler = 'vbla'
        export_filer = 'bla'
        connection_filer = 'blabla'
        username = 'fake'
        password = 12345
        vserver = 'vbla'

        self.client.filer_update(fakeid, vfiler, export_filer,
                                 connection_filer, username, password, vserver)
        mock_request.assert_called_with('/filers/%d' % fakeid, {
                                         'vfiler': vfiler,
                                         'connection_filer': connection_filer,
                                         'export_filer': export_filer,
                                         'username': username,
                                         'password': password,
                                         'vserver': vserver
                                        })

    # VOLUME
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._get_request')
    def test_volume_list(self, mock_request):

        self.client.volume_list()
        mock_request.assert_called_once_with('/volumes')

    @patch('faasclient.client.Client._post_request')
    def test_volume_create(self, mock_request):

        path = '/some/path'
        filer_id = 1
        categoria_id = 1
        maxsnapshots = 10

        self.client.volume_create(path, filer_id, categoria_id, maxsnapshots)
        mock_request.assert_called_once_with('/volumes', {
                                             'path': path,
                                             'filer_id': filer_id,
                                             'categoria_id': categoria_id,
                                             'maxsnapshots': maxsnapshots
                                             })

    @patch('faasclient.client.Client._get_request')
    def test_volume_get(self, mock_request):

        fakeid = 123

        self.client.volume_get(fakeid)
        mock_request.assert_called_once_with('/volumes/%d' % fakeid)

    @patch('faasclient.client.Client._delete_request')
    def test_volume_delete(self, mock_request):

        fakeid = 123

        self.client.volume_delete(fakeid)
        mock_request.assert_called_once_with('/volumes/%d' % fakeid)

    @patch('faasclient.client.Client._put_request')
    def test_volume_update(self, mock_request):

        fakeid = 123
        path = '/some/path'
        filer_id = 1
        categoria_id = 1
        maxsnapshots = 10

        self.client.volume_update(fakeid, path, filer_id, categoria_id,
                                  maxsnapshots)
        mock_request.assert_called_once_with('/volumes/%d' % fakeid, {
                                             'path': path,
                                             'filer_id': filer_id,
                                             'categoria_id': categoria_id,
                                             'maxsnapshots': maxsnapshots
                                             })

    @patch('faasclient.client.Client._get_request')
    def test_volumes_info(self, mock_request):

        self.client.volumes_info()
        mock_request.assert_called_once_with('/volumes-info')

    @patch('faasclient.client.Client._post_request')
    def test_volumes_update_info(self, mock_request):

        self.client.volumes_update_info()
        mock_request.assert_called_once_with('/volumes-info-update')

    # PERMISSAO
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._get_request')
    def test_permission_list(self, mock_request):

        self.client.permission_list()
        mock_request.assert_called_once_with('/permissions')

    @patch('faasclient.client.Client._post_request')
    def test_permission_create(self, mock_request):

        name = 'permissao'

        self.client.permission_create(name)
        mock_request.assert_called_once_with('/permissions', {'name': name})

    @patch('faasclient.client.Client._get_request')
    def test_permission_get(self, mock_request):

        id = 123

        self.client.permission_get(id)
        mock_request.assert_called_once_with('/permissions/%d' % id)

    @patch('faasclient.client.Client._put_request')
    def test_permission_update(self, mock_request):

        id = 123
        name = 'permissao'

        self.client.permission_update(id, name)
        mock_request.assert_called_once_with('/permissions/%d' % id, {
                                             'name': name
                                             })

    @patch('faasclient.client.Client._delete_request')
    def test_permission_delete(self, mock_request):

        id = 123

        self.client.permission_delete(id)
        mock_request.assert_called_once_with('/permissions/%d' % id)

    # QUOTA
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._get_request')
    def test_quota_get(self, mock_request):

        id = 123

        self.client.quota_get(id)
        mock_request.assert_called_once_with('/exports/%d/quota' % id)

    @patch('faasclient.client.Client._put_request')
    def test_quota_set(self, mock_request):

        id = 123
        quota = 123456789
        time_id = '123456'

        self.client.export_update(id, quota, time_id)
        mock_request.assert_called_once_with('/exports/%d' % id, {
                                             'quota': quota,
                                             'time_id': time_id
                                             }, audit=False)

    @patch('faasclient.client.Client._delete_request')
    def test_quota_delete(self, mock_request):

        id = 123

        self.client.quota_delete(id)
        mock_request.assert_called_once_with('/exports/%d/quota' % id)

    # AUDIT
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._get_request')
    def test_audit_list(self, mock_request):

        self.client.audit_list()
        mock_request.assert_called_once_with('/audits')

    @patch('faasclient.client.Client._get_request')
    def test_audit_get(self, mock_request):

        id = 123

        self.client.audit_get(id)
        mock_request.assert_called_once_with('/audits/%d' % id)

    @patch('faasclient.client.Client._get_request')
    def test_audit_export_get(self, mock_request):

        export_id = 123

        self.client.audit_export_get(export_id)
        mock_request.assert_called_once_with('/audits/export/%d' % export_id)

    # APPHOST
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._get_request')
    def test_apphost_list(self, mock_request):

        self.client.apphost_list()
        mock_request.assert_called_once_with('/apphosts')

    @patch('faasclient.client.Client._post_request')
    def test_apphost_create(self, mock_request):

        ip = '1.1.1.1'
        hostname = 'my-hostname'

        self.client.apphost_create(ip, hostname)
        mock_request.assert_called_once_with('/apphosts', {
                                             'ip': ip,
                                             'hostname': hostname})

    @patch('faasclient.client.Client._get_request')
    def test_apphost_get(self, mock_request):

        id = 123
        self.client.apphost_get(id)
        mock_request.assert_called_once_with('/apphosts/%d' % id)

    @patch('faasclient.client.Client._put_request')
    def test_apphost_update(self, mock_request):

        id = 123
        ip = '2.2.2.2'
        hostname = 'my-hostname2'

        self.client.apphost_update(id, ip, hostname)
        mock_request.assert_called_once_with('/apphosts/%d' % id, {
                                             'ip': ip,
                                             'hostname': hostname})

    @patch('faasclient.client.Client._delete_request')
    def test_apphost_delete(self, mock_request):

        id = 123
        self.client.apphost_delete(id)
        mock_request.assert_called_once_with('/apphosts/%d' % id)

    # RESTORE / JOBS
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._get_request')
    def test_jobs_restore_list(self, mock_request):
        self.client.jobs_list()
        mock_request.assert_called_once_with('/restore/jobs')

    @patch('faasclient.client.Client._get_request')
    def test_job_restore_get(self, mock_request):
        id = 123

        self.client.jobs_get(id)
        mock_request.assert_called_once_with('/restore/jobs/%d' % id)

    @patch('faasclient.client.Client._post_request')
    def test_restore_snapshot(self, mock_request):
        export_id = 123
        snapshot_id = 123

        self.client.snapshot_restore(export_id, snapshot_id)
        mock_request.assert_called_once_with('/exports/%d/snapshots/%d/restore' % (export_id, snapshot_id), audit=False)

    # ACCESS
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._get_request')
    def test_access_list(self, mock_request):
        export_id_or_path = 123

        self.client.access_list(export_id_or_path)
        mock_request.assert_called_once_with('/exports/%d/hosts' % export_id_or_path)

    @patch('faasclient.client.Client._post_request')
    def test_access_create(self, mock_request):

        export_id_or_path = 123
        permission = 'read-write'
        host = '127.0.0.1'

        parameters = {'host': '127.0.0.1', 'permission': 'read-write'}

        self.client.access_create(export_id_or_path, permission, host)
        mock_request.assert_called_once_with('/exports/%d/hosts' % export_id_or_path, parameters, audit=False)

    @patch('faasclient.client.Client._get_request')
    def test_access_get(self, mock_request):

        export_id_or_path = 1
        access_id = 1

        self.client.access_get(export_id_or_path, access_id)
        mock_request.assert_called_once_with('/exports/%d/hosts/%d' % (export_id_or_path, access_id))

    @patch('faasclient.client.Client._delete_request')
    def test_access_delete(self, mock_request):

        export_id_or_path = 1
        access_id = 2

        self.client.access_delete(export_id_or_path, access_id)
        mock_request.assert_called_once_with('/exports/%d/hosts/%d' % (export_id_or_path, access_id), audit=False)

    # SNAPSHOT
    # ---------------------------------------------------------------------
    @patch('faasclient.client.Client._get_request')
    def snapshot_list(self, mock_request):
        export_id_or_path = 123

        self.client.snapshot_list(export_id_or_path)
        mock_request.assert_called_once_with('/exports/%d/snapshots' % export_id_or_path)

    @patch('faasclient.client.Client._post_request')
    def test_snapshot_create(self, mock_request):

        export_id_or_path = 123

        self.client.snapshot_create(export_id_or_path)
        mock_request.assert_called_once_with('/exports/%d/snapshots' % export_id_or_path, audit=False)

    @patch('faasclient.client.Client._get_request')
    def test_snapshot_get(self, mock_request):

        export_id_or_path = 1
        snapshot_id = 2

        self.client.snapshot_get(export_id_or_path, snapshot_id)
        mock_request.assert_called_once_with('/exports/%d/snapshots/%d' % (export_id_or_path, snapshot_id))

    @patch('faasclient.client.Client._delete_request')
    def test_snapshot_delete(self, mock_request):

        export_id_or_path = 1
        snapshot_id = 2

        self.client.snapshot_delete(export_id_or_path, snapshot_id)
        mock_request.assert_called_once_with('/exports/%d/snapshots/%d' % (export_id_or_path, snapshot_id), audit=False)



# Copyright 2019 Globo.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
import json
import logging
import requests
from keystoneclient import exceptions as ksexceptions
from faasclient.exceptions import ClientException
try:
    from urlparse import urlparse
except ImportError:
    from urllib.parse import urlparse


try:
    from logging import NullHandler
except ImportError:
    # Added in Python 2.7
    class NullHandler(logging.Handler):
        def handle(self, record):
            pass

        def emit(self, record):
            pass

        def createLock(self):
            self.lock = None


logger = logging.getLogger('faasclient')


class Client(object):

    def __init__(self, authurl=None, user=None, key=None, preauthurl=None,
                 preauthtoken=None, tenant_name=None, os_options=None,
                 insecure=False):
        """
        :param authurl: authentication URL
        :param user: user name to authenticate as
        :param key: key/password to authenticate with
        :param retries: Number of times to retry the request before failing
        :param preauthurl: storage URL (if you have already authenticated)
        :param preauthtoken: authentication token (if you have already
                             authenticated) note authurl/user/key/tenant_name
                             are not required when specifying preauthtoken
        :param tenant_name: The tenant/account name, required when connecting
                            to an auth 2.0 system.
        :param os_options: The OpenStack options which can have tenant_id,
                           auth_token, service_type, endpoint_type,
                           tenant_name, block_storage_url, region_name,
                           service_username, service_project_name, service_key
        :param insecure: Allow to access servers without checking SSL certs.
                         The server's certificate will not be verified.
        """
        self.authurl = authurl
        self.user = user
        self.key = key
        self.http_conn = None
        self.attempts = 0
        self.os_options = dict(os_options or {})
        if tenant_name:
            self.os_options['tenant_name'] = tenant_name
        if preauthurl:
            self.os_options['block_storage_url'] = preauthurl

        self.url = preauthurl or self.os_options.get('block_storage_url')
        self.token = preauthtoken or self.os_options.get('auth_token')

        if self.os_options.get('service_username', None):
            self.service_auth = True
        else:
            self.service_auth = False
        self.service_token = None
        self.insecure = insecure

    # AUTENTICACAO  ------------------------------------------------------------
    def get_auth(self):
        url, token = get_keystone_auth(self.authurl, self.user, self.key,
                                       os_options=self.os_options,
                                       insecure=self.insecure)
        return url, token

    # ACCESS  ------------------------------------------------------------------
    def access_list(self, export_id_or_path):
        url_path = '/exports/{}/hosts'.format(export_id_or_path)

        return self._get_request(url_path)

    def access_delete(self, export_id_or_path, access_id, audit=False):
        url_path = '/exports/{}/hosts/{}'.format(export_id_or_path, access_id)

        return self._delete_request(url_path, audit=audit)

    def access_get(self, export_id_or_path, access_id):
        url_path = '/exports/{}/hosts/{}'.format(export_id_or_path, access_id)

        return self._get_request(url_path)

    def access_create(self, export_id_or_path, permission, host, audit=False):
        url_path = '/exports/{}/hosts'.format(export_id_or_path)
        data = {'permission': permission, 'host': host}

        return self._post_request(url_path, data, audit=audit)

    # CATEGORIA  ---------------------------------------------------------------
    def categoria_list(self):
        url_path = '/categorias'

        return self._get_request(url_path)

    def categoria_get(self, categoria_id):
        url_path = '/categorias/{}'.format(categoria_id)

        return self._get_request(url_path)

    def categoria_create(self, name):
        url_path = '/categorias'
        data = {'name': name}

        return self._post_request(url_path, data)

    def categoria_update(self, categoria_id, name):
        url_path = '/categorias/{}'.format(categoria_id)
        data = {'name': name}

        return self._put_request(url_path, data)

    def categoria_delete(self, categoria_id):
        url_path = '/categorias/{}'.format(categoria_id)

        return self._delete_request(url_path)

    # AUDIT  -------------------------------------------------------------------
    def audit_list(self):
        url_path = '/audits'

        return self._get_request(url_path)

    def audit_get(self, audit_id):
        url_path = '/audits/{}'.format(audit_id)

        return self._get_request(url_path)

    def audit_export_get(self, export_id):
        url_path = '/audits/export/{}'.format(export_id)

        return self._get_request(url_path)

    def audit_custeio_create(self, id_custeio, mensagem, abertura, fechamento, json_custeio):
        url_path = '/custeio_audit'
        data = {'id_custeio': id_custeio, 'mensagem': mensagem, 'abertura': abertura, 'fechamento': fechamento, 'json_custeio': json_custeio}

        return self._post_request(url_path, data)

    # APPHOST  ------------------------------------------------------------
    def apphost_list(self):
        url_path = '/apphosts'

        return self._get_request(url_path)

    def apphost_create(self, ip, hostname):
        url_path = '/apphosts'
        data = {'ip': ip, 'hostname': hostname}

        return self._post_request(url_path, data)

    def apphost_get(self, conf_id):
        url_path = '/apphosts/{}'.format(conf_id)
        return self._get_request(url_path)

    def apphost_delete(self, conf_id):
        url_path = '/apphosts/{}'.format(conf_id)

        return self._delete_request(url_path)

    def apphost_update(self, conf_id, ip, hostname):
        url_path = '/apphosts/{}'.format(conf_id)
        data = {'ip': ip, 'hostname': hostname}

        return self._put_request(url_path, data)

    # EXPORT  ------------------------------------------------------------------
    def project_quota_get(self):
        url_path = '/quota'
        return self._get_request(url_path)

    def export_get(self, export_id_or_path):
        url_path = '/exports/{}'.format(export_id_or_path)
        return self._get_request(url_path)

    def export_list(self, host=None, extra=None):
        url_path = '/exports'
        params = {}
        query_str = ''

        if host:
            params['host'] = host

        if extra:
            params['extra'] = extra

        for i, key in enumerate(params.keys()):
            if i > 0:
                query_str += '&'
            query_str += '{}={}'.format(key, params[key])

        if len(query_str) > 0:
            url_path = '{}?{}'.format(url_path, query_str)

        return self._get_request(url_path)

    def deleted_export_list(self):
        url_path = '/exports/deleted'
        return self._get_request(url_path)

    def export_create(self, quota, categoria, time_id, resource_id=None, audit=False):
        url_path = '/exports'
        data = {'quota': quota, 'categoria': categoria, 'time_id': time_id, 'resource_id': resource_id}

        return self._post_request(url_path, data, audit=audit)

    def export_multiple_create(self, quota, categoria, time_id, resource_id=None, amount=None, audit=False):
        url_path = '/exports/multiple'
        data = {'quota': quota, 'categoria': categoria, 'time_id': time_id, 'resource_id': resource_id, 'amount': amount}

        return self._post_request(url_path, data, audit=audit)

    def export_delete(self, export_id_or_path, audit=False):
        url_path = '/exports/{}'.format(export_id_or_path)
        return self._delete_request(url_path, audit=audit)

    def export_force_delete(self, export_id_or_path, audit=False):
        url_path = '/exports/{}/force'.format(export_id_or_path)
        return self._delete_request(url_path, audit=audit)

    def export_update(self, export_id_or_path, quota, time_id=None, audit=False):
        url_path = '/exports/{}'.format(export_id_or_path)
        data = {'quota': quota, 'time_id': time_id}

        return self._put_request(url_path, data, audit=audit)

    def export_group(self, exports_id, resource_id, audit=False):
        url_path = '/exports/resource'
        data = {'exports': exports_id, 'resource_id': resource_id}

        return self._post_request(url_path, data, audit=audit)

    def exports_time(self, exports_id, time_id, audit=False):
        url_path = '/exports/time'
        data = {'exports': exports_id, 'time_id': time_id}

        return self._post_request(url_path, data, audit=audit)

    def export_by_id_or_path(self, export_id_or_path):
        url_path = '/notenant/exports/{}'.format(export_id_or_path)
        return self._get_request(url_path)

    def export_all_list(self):
        url_path = '/notenant/exports'
        return self._get_request(url_path)

    # FILER  ------------------------------------------------------------------
    def filer_list(self):
        url_path = '/filers'

        return self._get_request(url_path)

    def filer_create(self, vfiler, export_filer, connection_filer, username,
                     password, vserver):
        url_path = '/filers'
        data = {
            'vfiler': vfiler,
            'export_filer': export_filer,
            'connection_filer': connection_filer,
            'username': username,
            'password': password,
            'vserver': vserver,
        }

        return self._post_request(url_path, data)

    def filer_get(self, filer_id):
        url_path = '/filers/{}'.format(filer_id)

        return self._get_request(url_path)

    def filer_delete(self, filer_id):
        url_path = '/filers/{}'.format(filer_id)

        return self._delete_request(url_path)

    def filer_update(self, filer_id, vfiler, export_filer, connection_filer,
                     username, password, vserver):
        url_path = '/filers/{}'.format(filer_id)
        data = {
            'vfiler': vfiler,
            'export_filer': export_filer,
            'connection_filer': connection_filer,
            'username': username,
            'password': password,
            'vserver': vserver,
        }

        return self._put_request(url_path, data)

    # VOLUME  ------------------------------------------------------------------
    def volume_list(self):
        url_path = '/volumes'

        return self._get_request(url_path)

    def volume_create(self, path, filer_id, categoria_id, maxsnapshots):
        url_path = '/volumes'
        data = {
            'path': path,
            'filer_id': filer_id,
            'categoria_id': categoria_id,
            'maxsnapshots': maxsnapshots,
        }

        return self._post_request(url_path, data)

    def volume_get(self, volume_id):
        url_path = '/volumes/{}'.format(volume_id)

        return self._get_request(url_path)

    def volume_delete(self, volume_id):
        url_path = '/volumes/{}'.format(volume_id)

        return self._delete_request(url_path)

    def volume_update(self, volume_id, path, filer_id, categoria_id,
                      maxsnapshots):
        url_path = '/volumes/{}'.format(volume_id)
        data = {
            'path': path,
            'filer_id': filer_id,
            'categoria_id': categoria_id,
            'maxsnapshots': maxsnapshots,
        }

        return self._put_request(url_path, data)

    def volumes_info(self):
        url_path = '/volumes-info'

        return self._get_request(url_path)

    def volumes_update_info(self):
        url_path = '/volumes-info-update'

        return self._post_request(url_path)

    # PERMISSAO  ---------------------------------------------------------------
    def permission_list(self):
        url_path = '/permissions'

        return self._get_request(url_path)

    def permission_create(self, name):
        url_path = '/permissions'
        data = {'name': name}

        return self._post_request(url_path, data)

    def permission_get(self, permission_id):
        url_path = '/permissions/{}'.format(permission_id)
        return self._get_request(url_path)

    def permission_delete(self, permission_id):
        url_path = '/permissions/{}'.format(permission_id)

        return self._delete_request(url_path)

    def permission_update(self, permission_id, name):
        url_path = '/permissions/{}'.format(permission_id)
        data = {'name': name}

        return self._put_request(url_path, data)

    # QUOTA  -------------------------------------------------------------------
    def quota_get(self, export_id=None):
        if export_id:
            url_path = '/exports/{}/quota'.format(export_id)
        else:
            url_path = '/quota'

        return self._get_request(url_path)

    def quota_post(self, export_id, size):
        url_path = '/exports/{}/quota'.format(export_id)
        data = {'size': size}

        return self._post_request(url_path, data)

    def quota_delete(self, export_id):
        url_path = '/exports/{}/quota'.format(export_id)

        return self._delete_request(url_path)

    # RESTORE / JOBS  ----------------------------------------------------------
    def jobs_list(self):
        url_path = '/restore/jobs'

        return self._get_request(url_path)

    def jobs_get(self, job_id):
        url_path = '/restore/jobs/{}'.format(job_id)

        return self._get_request(url_path)

    # SNAPSHOTS  ---------------------------------------------------------------
    def snapshot_list(self, export_id_or_path):
        url_path = '/exports/{}/snapshots'.format(export_id_or_path)

        return self._get_request(url_path)

    def snapshot_delete(self, export_id_or_path, snapshot_id, audit=False):
        url_path = '/exports/{}/snapshots/{}'.format(export_id_or_path, snapshot_id)

        return self._delete_request(url_path, audit=audit)

    def snapshot_get(self, export_id_or_path, snapshot_id):
        url_path = '/exports/{}/snapshots/{}'.format(export_id_or_path, snapshot_id)

        return self._get_request(url_path)

    def snapshot_create(self, export_id_or_path, audit=False):
        url_path = '/exports/{}/snapshots'.format(export_id_or_path)

        return self._post_request(url_path, audit=audit)

    def snapshot_restore(self, export_id_or_path, snapshot_id, audit=False):
        url_path = '/exports/{}/snapshots/{}/restore'.format(export_id_or_path, snapshot_id)

        return self._post_request(url_path, audit=audit)

    # REPORT TO GLOBOMAP  ---------------------------------------------------------------
    def report_get(self):
        url_path = '/report'

        return self._get_request(url_path)

    # REQUESTS
    def _request(self, method, url_path, data=None, headers=None, audit=False):
        if not self.token:
            project_url, self.token = self.get_auth()

        if self.url is None:
            self.url = project_url

        headers = headers or {}  # Case headers is None
        headers['X-Storage-Token'] = self.token
        url = self.url + url_path

        if '/notenant' in url_path:
            parsed = urlparse(self.url)
            url_path = ''.join(url_path.split('/notenant')[1:])

            url = '{}://{}{}'.format(parsed.scheme, parsed.hostname, url_path)

        if url_path == '/report':
            parsed = urlparse(self.url)

            url = '{}://{}{}'.format(parsed.scheme, parsed.hostname, url_path)

        r = requests.request(method, url, data=data, headers=headers,
                             verify=not self.insecure, timeout=1800)

        if audit:
            return r

        try:
            response = r.json()
        except ValueError:
            response = ''

        return r.status_code, response

    def _get_request(self, url_path, headers=None):
        return self._request('get', url_path, headers=headers)

    def _post_request(self, url_path, data=None, headers=None, audit=False):

        return self._request('post', url_path, data=data, headers=headers, audit=audit)

    def _put_request(self, url_path, data, headers=None, audit=False):
        return self._request('put', url_path, data=data, headers=headers, audit=audit)

    def _delete_request(self, url_path, headers=None, audit=False):
        return self._request('delete', url_path, headers=headers, audit=audit)


def _import_keystone_client(auth_version):
    """
    Import the proper keystoneclient for a specific version. If a invalid
    version is requests, version 2 will be loaded.

    Based on:
    https://github.com/openstack/python-swiftclient/blob/
    307d4c007afaccdcd70628d9fce44231115d62cd/swiftclient/client.py#L522
    """

    try:
        if auth_version in ('3.0', '3', 3):
            from keystoneclient.v3 import client as ksclient
        else:
            from keystoneclient.v2_0 import client as ksclient

        # prevent keystoneclient warning us that it has no log handlers
        logging.getLogger('keystoneclient').addHandler(NullHandler())

        return ksclient
    except ImportError:
        raise ClientException('''
Auth versions 2.0 and 3 require python-keystoneclient, install it or use Auth
version 1.0 which requires ST_AUTH, ST_USER, and ST_KEY environment
variables to be set or overridden with -A, -U, or -K.''')


def _discover_auth_version(auth_url):
    # Return the auth version (2 or 3) based on auth_url parameter
    # Ex.: https://domain:port/v2.0 => 2
    #      https://domain:port/v3   => 3

    if auth_url is None:
        raise ClientException('Unknown Auth URL')

    version = 'v2.0'
    try:
        version = auth_url.split('/')[3]
    except IndexError:
        ClientException('Unknown Auth version')

    if version in ('v3', 'v3.0', '3'):
        return 3

    return 2


def get_keystone_auth(auth_url, user, key, os_options, **kwargs):
    """
    Authenticate against a keystone server.
    We are using the keystoneclient library for authentication.
    """

    insecure = kwargs.get('insecure', False)
    timeout = kwargs.get('timeout', None)
    debug = logger.isEnabledFor(logging.DEBUG)

    auth_version = _discover_auth_version(auth_url)

    ksclient = _import_keystone_client(auth_version)

    try:
        _ksclient = ksclient.Client(
            username=user,
            password=key,
            token=os_options.get('auth_token'),
            tenant_name=os_options.get('tenant_name'),
            tenant_id=os_options.get('tenant_id'),
            user_id=os_options.get('user_id'),
            user_domain_name=os_options.get('user_domain_name'),
            user_domain_id=os_options.get('user_domain_id'),
            project_name=os_options.get('project_name'),
            project_id=os_options.get('project_id'),
            project_domain_name=os_options.get('project_domain_name'),
            project_domain_id=os_options.get('project_domain_id'),
            debug=debug,
            auth_url=auth_url, insecure=insecure, timeout=timeout)
    except ksexceptions.Unauthorized:
        msg = 'Unauthorized. Check username, password and tenant name/id.'
        raise ClientException(msg)
    except ksexceptions.AuthorizationFailure as err:
        raise ClientException('Authorization Failure. %s' % err)

    service_type = os_options.get('service_type') or 'block-storage'
    endpoint_type = os_options.get('endpoint_type') or 'adminURL'

    try:
        filter_kwargs = {}
        if os_options.get('region_name'):
            filter_kwargs['attr'] = 'region'
            filter_kwargs['filter_value'] = os_options['region_name']

        endpoint = _ksclient.service_catalog.url_for(
            service_type=service_type,
            endpoint_type=endpoint_type,
            **filter_kwargs)
    except ksexceptions.EndpointNotFound:
        raise ClientException('Endpoint for %s not found - '
                              'have you specified a region?' % service_type)

    return endpoint, _ksclient.auth_token



# Copyright 2018 Globo.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" Tests for the faasclient.client.get_keystone_auth method """

import sys
import unittest

from keystoneclient import exceptions as ksexceptions
from mock import patch, Mock

from faasclient.client import get_keystone_auth, _discover_auth_version, \
    _import_keystone_client
from faasclient.exceptions import ClientException


class TestGetAuth(unittest.TestCase):

    def setUp(self):
        self.ksclient = Mock()
        patch('faasclient.client._import_keystone_client', Mock(
            return_value=self.ksclient)).start()

    def tearDown(self):
        patch.stopall()

    def test_get_keystone_auth_called_with_minimum_parameters(self):
        """
        Verify if get_keystone_auth calls keystone.Client with right parameters
        """

        get_keystone_auth('https://localhost/v2.0', 'user', 'key', {})

        self.ksclient.Client.assert_called_with(
            username='user',
            password='key',
            token=None,
            tenant_name=None,
            tenant_id=None,
            user_id=None,
            user_domain_name=None,
            user_domain_id=None,
            project_name=None,
            project_id=None,
            project_domain_name=None,
            project_domain_id=None,
            debug=False,
            auth_url='https://localhost/v2.0',
            insecure=False,
            timeout=None)

        service_catalog = self.ksclient.Client.return_value.service_catalog
        service_catalog.url_for.assert_called_with(
            service_type='block-storage',
            endpoint_type='adminURL')

    def test_get_keystone_auth_called_with_all_parameters(self):
        """
        Verify if get_keystone_auth calls keystone.Client with right parameters
        """

        os_options = {
            'token': '1234567890',
            'tenant_name': 'tenant_name',
            'tenant_id': '11223344',
            'user_id': '44112233',
            'user_domain_name': 'user_domain_name',
            'user_domain_id': '0987654321',
            'project_name': 'project_name',
            'project_id': '12211221',
            'project_domain_name': 'project_domain_name',
            'project_domain_id': '45454545',
        }

        get_keystone_auth('https://localhost/v2.0', 'user', 'key', os_options,
                          insecure=True, timeout=5)

        self.ksclient.Client.assert_called_with(
            username='user',
            password='key',
            token=os_options.get('auth_token'),
            tenant_name=os_options.get('tenant_name'),
            tenant_id=os_options.get('tenant_id'),
            user_id=os_options.get('user_id'),
            user_domain_name=os_options.get('user_domain_name'),
            user_domain_id=os_options.get('user_domain_id'),
            project_name=os_options.get('project_name'),
            project_id=os_options.get('project_id'),
            project_domain_name=os_options.get('project_domain_name'),
            project_domain_id=os_options.get('project_domain_id'),
            debug=False,
            auth_url='https://localhost/v2.0',
            insecure=True,
            timeout=5)

    def test_get_keystone_auth_return_values(self):
        """
        Verify if get_keystone_auth returns the proper token and endpoint
        """

        expected_endpoint = 'https://localhost:5000/v2.0'
        expected_token = '1234567890'

        self.ksclient.Client.return_value.auth_token = expected_token
        self.ksclient.Client.return_value.service_catalog.url_for\
            .return_value = expected_endpoint

        computed_enpoint, computed_token = get_keystone_auth(
            'https://localhost:5000/v2.0', 'user', 'key', {})

        self.assertEqual(expected_endpoint, computed_enpoint)
        self.assertEqual(expected_token, computed_token)

    def test_get_keystone_auth_not_authorized(self):
        """
        Verify if get_keystone_auth raises ClientException with the proper
        message when the authentication is denied
        """

        self.ksclient.Client.side_effect = ksexceptions.Unauthorized

        with self.assertRaises(ClientException) as excpt:
            get_keystone_auth('https://localhost/v2.0', 'user', 'key', {})

        expected = 'Unauthorized. Check username, password and tenant name/id.'
        computed = str(excpt.exception)

        self.assertEqual(expected, computed)

    def test_get_keystone_auth_authorization_failure(self):
        """
        Verify if get_keystone_auth raises ClientException with the proper
        message when the authentication fails
        """

        self.ksclient.Client.side_effect = ksexceptions.AuthorizationFailure(
            'Falha!')

        with self.assertRaises(ClientException) as excpt:
            get_keystone_auth('https://localhost/v2.0', 'user', 'key', {})

        expected = 'Authorization Failure. Falha!'
        computed = str(excpt.exception)

        self.assertEqual(expected, computed)

    def test_get_keystone_auth_endpoint_not_found(self):
        """
        Verify if get_keystone_auth raises ClientException if the
        os_options['endpoint_type'] and/or the os_options['service_type'] do
        not exists in the auth response
        """

        self.ksclient.Client.return_value\
            .service_catalog\
            .url_for\
            .side_effect = ksexceptions.EndpointNotFound

        with self.assertRaises(ClientException) as excpt:
            get_keystone_auth('https://localhost/v2.0', 'user', 'key', {})

        expected = ('Endpoint for block-storage not found'
                    ' - have you specified a region?')
        computed = str(excpt.exception)

        self.assertEqual(expected, computed)

    def test_discover_auth_version(self):

        version = _discover_auth_version('https://localhost:5000/v2.0')
        self.assertEqual(version, 2)

        version = _discover_auth_version('https://localhost:5000/v3')
        self.assertEqual(version, 3)

    def test_import_keystone_client_v2(self):

        _import_keystone_client(2)
        self.assertIn('keystoneclient.v2_0', sys.modules)

    def test_import_keystone_client_v3(self):

        _import_keystone_client(3)
        self.assertIn('keystoneclient.v3', sys.modules)

    def test_import_keystone_client_invalid_version(self):
        """ Default version loaded is v2 """

        _import_keystone_client('invalid')
        self.assertIn('keystoneclient.v2_0', sys.modules)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import os

from setuptools import setup, find_packages


def get_version(package):
    """Return package version as listed in `__version__` in `init.py`."""
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("__version__ = ['\"]([^'\"]+)['\"]", init_py).group(1)


setup(
    name="python-faasclient",
    url="https://gitlab.globoi.com/storm/python-faasclient",
    version=get_version("faasclient"),
    description='Python bindings to the Filer as a Service API.',
    author='STORM',
    author_email='storm@corp.globo.com',
    packages=find_packages(),
    install_requires=[
        'click>=6.2',
        'prettytable==0.7.2',
        'python-keystoneclient>=3.16.0',
        'requests>=2.19.1'
    ],
    entry_points={
        'console_scripts': [
            'faas = bin.commands:cli',
        ],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],
    zip_safe=False,
)



# Copyright 2019 Globo.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import click
import prettytable
import json

from faasclient import client
from requests.packages import urllib3


# Disable HTTPS verification warnings.
urllib3.disable_warnings()


OS_USERNAME_HELP = ('Name used for authentication with the OpenStack Identity '
                    'service. Defaults to env[OS_USERNAME].')

OS_PASSWORD_HELP = ('Password used for authentication with the OpenStack '
                    'Identity service. Defaults to env[OS_PASSWORD].')

OS_TENANT_NAME_HELP = ('Tenant to request authorization on. Defaults to '
                       'env[OS_TENANT_NAME].')

OS_AUTH_URL_HELP = ('Specify the Identity endpoint to use for authentication. '
                    'Defaults to env[OS_AUTH_URL].')


@click.group(context_settings={'help_option_names': ['-h', '--help']})
@click.option('--os-username', envvar='OS_USERNAME', help=OS_USERNAME_HELP,
              metavar='<auth-user-name>')
@click.option('--os-password', envvar='OS_PASSWORD', help=OS_PASSWORD_HELP,
              metavar='<auth-password>')
@click.option('--os-tenant-name', envvar='OS_TENANT_NAME',
              help=OS_TENANT_NAME_HELP, metavar='<auth-tenant-name>')
@click.option('--os-auth-url', envvar='OS_AUTH_URL', help=OS_AUTH_URL_HELP,
              metavar='<auth-url>')
@click.pass_context
def cli(ctx, os_username, os_password, os_tenant_name, os_auth_url):

    conn = client.Client(authurl=os_auth_url,
                         user=os_username,
                         key=os_password,
                         tenant_name=os_tenant_name,
                         insecure=True)

    ctx.obj = {'CONN': conn}


# ACCESS ----------------------------------------------------------------------
@cli.command(name='access-list')
@click.option('export_id', '--export-id', metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.pass_context
def access_list(ctx, export_id):
    conn = ctx.obj['CONN']
    print_response(conn.access_list(export_id))


@cli.command(name='access-get')
@click.option('--export-id', metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.option('--access-id', metavar='<access_id>',
              type=click.INT, required=True)
@click.pass_context
def access_get(ctx, export_id, access_id):
    conn = ctx.obj['CONN']
    print_response(conn.access_get(export_id, access_id))


@cli.command(name='access-create')
@click.option('--export-id', metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.option('--permission', metavar='<permission>', type=click.STRING,
              required=True)
@click.option('--host', metavar='<host>', type=click.STRING, required=True)
@click.pass_context
def access_create(ctx, export_id, permission, host):
    conn = ctx.obj['CONN']
    print_response(conn.access_create(export_id, permission, host))


@cli.command(name='access-delete')
@click.option('--export-id', metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.option('--access-id', metavar='<access_id>',
              type=click.INT, required=True)
@click.pass_context
def access_delete(ctx, export_id, access_id):
    conn = ctx.obj['CONN']
    print_response(conn.access_delete(export_id, access_id))


# CATEGORIA -------------------------------------------------------------------
@cli.command(name='categoria-list')
@click.pass_context
def categoria_list(ctx):
    conn = ctx.obj['CONN']
    print_response(conn.categoria_list())


@cli.command(name='categoria-create')
@click.option('--name', metavar='<categoria-name>', type=click.STRING,
              required=True)
@click.pass_context
def categoria_create(ctx, name):
    conn = ctx.obj['CONN']
    print_response(conn.categoria_create(name))


@cli.command(name='categoria-get')
@click.option('categoria_id', '--id', metavar='<categoria_id>', type=click.INT,
              required=True)
@click.pass_context
def categoria_get(ctx, categoria_id):
    conn = ctx.obj['CONN']
    print_response(conn.categoria_get(categoria_id))


@cli.command(name='categoria-update')
@click.option('categoria_id', '--id', metavar='<categoria_id>', type=click.INT,
              required=True)
@click.option('--name', metavar='<categoria_name>', type=click.STRING,
              required=True)
@click.pass_context
def categoria_update(ctx, categoria_id, name):
    conn = ctx.obj['CONN']
    print_response(conn.categoria_update(categoria_id, name))


@cli.command(name='categoria-delete')
@click.option('categoria_id', '--id', metavar='<categoria_id>', type=click.INT,
              required=True)
@click.pass_context
def categoria_delete(ctx, categoria_id):
    conn = ctx.obj['CONN']
    print_response(conn.categoria_delete(categoria_id))


# AUDIT ------------------------------------------------------------------------
@cli.command(name='audit-list')
@click.pass_context
def audit_list(ctx):
    conn = ctx.obj['CONN']
    print_response(conn.audit_list())


@cli.command(name='audit-get')
@click.option('audit_id', '--id', metavar='<audit_id>', type=click.INT,
              required=True)
@click.pass_context
def audit_get(ctx, audit_id):
    conn = ctx.obj['CONN']
    print_response(conn.audit_get(audit_id))


@cli.command(name='audit-export-get')
@click.option('export_id', '--export-id', metavar='<export_id>', type=click.INT,
              required=True)
@click.pass_context
def audit_export_get(ctx, export_id):
    conn = ctx.obj['CONN']
    print_response(conn.audit_export_get(export_id))


# id_custeio=1&mensagem=Custeio Inserido Com Sucesso&abertura=2019-07-16-00:00:00&fechamento=2019-07-16-23:59:59&json_custeio={teste:teste}
@cli.command(name='audit-custeio-create')
@click.option('--id_custeio', metavar='<id_custeio>', type=click.STRING,
              required=True)
@click.option('--mensagem', metavar='<mensagem>', type=click.STRING,
              required=True)
@click.option('--abertura', metavar='<abertura>', type=click.STRING,
              required=True)
@click.option('--fechamento', metavar='<fechamento>', type=click.STRING,
              required=True)
@click.option('--json_custeio', metavar='<json_custeio>',
              type=click.STRING, required=True)
@click.pass_context
def audit_custeio_create(ctx, id_custeio, mensagem, abertura, fechamento, json_custeio):
    conn = ctx.obj['CONN']
    print_response(conn.audit_custeio_create(id_custeio, mensagem, abertura, fechamento, json_custeio))


# apphost ----------------------------------------------------------------
@cli.command(name='apphost-list')
@click.pass_context
def apphost_list(ctx):
    conn = ctx.obj['CONN']
    print_response(conn.apphost_list())


@cli.command(name='apphost-create')
@click.option('--ip', metavar='<apphost_ip>', type=click.STRING,
              required=True)
@click.option('--hostname', metavar='<apphost_hostname>',
              type=click.INT, required=True)
@click.pass_context
def apphost_create(ctx, ip, hostname):
    conn = ctx.obj['CONN']
    print_response(conn.apphost_create(ip, hostname))


@cli.command(name='apphost-update')
@click.option('apphost_id', '--id', metavar='<apphost_id>',
              type=click.INT, required=True)
@click.option('--ip', metavar='<apphost_ip>', type=click.STRING,
              required=True)
@click.option('--hostname', metavar='<apphost_hostname>',
              type=click.STRING, required=True)
@click.pass_context
def apphost_update(ctx, apphost_id, ip, hostname):
    conn = ctx.obj['CONN']
    print_response(conn.apphost_update(apphost_id, ip, hostname))


@cli.command(name='apphost-get')
@click.option('apphost_id', '--id', metavar='<apphost_id>',
              type=click.INT, required=True)
@click.pass_context
def apphost_get(ctx, apphost_id):
    conn = ctx.obj['CONN']
    print_response(conn.apphost_get(apphost_id))


@cli.command(name='apphost-delete')
@click.option('apphost_id', '--id', metavar='<apphost_id>',
              type=click.INT, required=True)
@click.pass_context
def apphost_delete(ctx, apphost_id):
    conn = ctx.obj['CONN']
    print_response(conn.apphost_delete(apphost_id))


# EXPORT ----------------------------------------------------------------------
@cli.command(name='export-list')
@click.option('host', '--host', metavar='<host>', type=click.STRING,
              required=False)
@click.pass_context
def export_list(ctx, host):
    conn = ctx.obj['CONN']
    print_response(conn.export_list(host))


@cli.command(name='export-list-deleted')
@click.pass_context
def delete_export_list(ctx):
    conn = ctx.obj['CONN']
    print_response(conn.deleted_export_list())


@cli.command(name='project-quota-get')
@click.pass_context
def export_get(ctx):
    conn = ctx.obj['CONN']
    print_response(conn.project_quota_get())


@cli.command(name='export-get')
@click.option('export_id', '--id', metavar='<export_id or export_path>',
              type=click.STRING, required=True)
@click.pass_context
def export_get(ctx, export_id):
    conn = ctx.obj['CONN']
    print_response(conn.export_get(export_id))


@cli.command(name='export-create')
@click.option('--quota', metavar='<size in KB>', type=click.INT, required=True)
@click.option('--categoria', metavar='<categoria>', type=click.INT,
              required=True)
@click.option('--resource_id', metavar='<resource_id>', type=click.STRING,
              required=False)
@click.option('--time-id', metavar='<time_id>', type=click.STRING,
              required=False)
@click.pass_context
def export_create(ctx, quota, categoria, resource_id, time_id):
    conn = ctx.obj['CONN']
    print_response(conn.export_create(quota, categoria, resource_id, time_id))


@cli.command(name='export-multiple-create')
@click.option('--quota', metavar='<size in KB>', type=click.INT, required=True)
@click.option('--categoria', metavar='<categoria>', type=click.INT,
              required=True)
@click.option('--resource_id', metavar='<resource_id>', type=click.STRING,
              required=False)
@click.option('--time-id', metavar='<time_id>', type=click.STRING,
              required=False)
@click.option('--amount', metavar='<amount>', type=click.INT, required=True)
@click.pass_context
def export_multiple_create(ctx, quota, categoria, resource_id, time_id, amount):
    conn = ctx.obj['CONN']
    print_response(conn.export_multiple_create(quota, categoria, resource_id, time_id, amount))


@cli.command(name='export-delete')
@click.option('export_id', '--id', metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.option('--force', is_flag=True)
@click.pass_context
def export_delete(ctx, export_id, force):
    conn = ctx.obj['CONN']
    if force:
        print_response(conn.export_force_delete(export_id))
    else:
        print_response(conn.export_delete(export_id))


@cli.command(name='export-delete')
@click.option('export_id', '--id', metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.option('--force', is_flag=True)
@click.pass_context
def export_delete(ctx, export_id, force):
    conn = ctx.obj['CONN']
    if force:
        print_response(conn.export_force_delete(export_id))
    else:
        print_response(conn.export_delete(export_id))


@cli.command(name='group-exports')
@click.option('exports_id', '--id', multiple=True, metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.option('resource_id', '--resource-id', metavar='<resource_id>',
              type=click.STRING, required=True)
@click.pass_context
def export_group(ctx, exports_id, resource_id):
    conn = ctx.obj['CONN']
    print_response(conn.export_group(exports_id, resource_id))


@cli.command(name='time-exports')
@click.option('exports_id', '--id', multiple=True, metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.option('time_id', '--time-id', metavar='<time_id>',
              type=click.STRING, required=True)
@click.pass_context
def exports_time(ctx, exports_id, time_id):
    conn = ctx.obj['CONN']
    print_response(conn.exports_time(exports_id, time_id))


# FILER -----------------------------------------------------------------------
@cli.command(name='filer-list')
@click.pass_context
def filer_list(ctx):
    conn = ctx.obj['CONN']
    print_response(conn.filer_list())


@cli.command(name='filer-create')
@click.option('--vfiler', metavar='<vfiler>', type=click.STRING, required=True)
@click.option('--export-filer', metavar='<export_filer>', type=click.STRING,
              required=True)
@click.option('connection_filer', '--conn', metavar='<connection_filer>',
              type=click.STRING, required=True)
@click.option('--username', metavar='<username>', type=click.STRING,
              required=True)
@click.option('--password', metavar='<password>', type=click.STRING,
              required=True)
@click.pass_context
def filer_create(ctx, vfiler, export_filer, connection_filer, username,
                 password):
    conn = ctx.obj['CONN']
    response = conn.filer_create(vfiler, export_filer, connection_filer,
                                 username, password)
    print_response(response)


@cli.command(name='filer-get')
@click.option('filer_id', '--id', metavar='<filer_id>', required=True)
@click.pass_context
def filer_get(ctx, filer_id):
    conn = ctx.obj['CONN']
    print_response(conn.filer_get(filer_id))


@cli.command(name='filer-delete')
@click.option('filer_id', '--id', metavar='<filer_id>', required=True)
@click.pass_context
def filer_delete(ctx, filer_id):
    conn = ctx.obj['CONN']
    print_response(conn.filer_delete(filer_id))


@cli.command(name='filer-update')
@click.option('filer_id', '--id', metavar='<filer_id>', type=click.INT,
              required=True)
@click.option('--vfiler', metavar='<vfiler>', type=click.STRING, required=True)
@click.option('--export-filer', metavar='<export_filer>', type=click.STRING,
              required=True)
@click.option('connection_filer', '--conn', metavar='<connection_filer>',
              type=click.STRING, required=True)
@click.option('--username', metavar='<username>', type=click.STRING,
              required=True)
@click.option('--password', metavar='<password>', type=click.STRING,
              required=True)
@click.pass_context
def filer_update(ctx, filer_id, vfiler, export_filer, connection_filer,
                 username, password):
    conn = ctx.obj['CONN']
    response = conn.filer_update(filer_id, vfiler, export_filer, conn,
                                 username,
                                 password)
    print_response(response)


# VOLUME ----------------------------------------------------------------------
@cli.command(name='volume-list')
@click.pass_context
def volume_list(ctx):
    conn = ctx.obj['CONN']
    print_response(conn.volume_list())


@cli.command(name='volume-create')
@click.option('--path', metavar='<path>', type=click.STRING, required=True)
@click.option('filer_id', '--filer-id', metavar='<filer_id>', type=click.INT,
              required=True)
@click.option('categoria_id', '--categoria-id', metavar='<categoria_id>',
              type=click.INT, required=True)
@click.option('--max', metavar='<maxsnapshots>', type=click.INT, required=True)
@click.pass_context
def volume_create(ctx, path, filer_id, categoria_id, max):
    conn = ctx.obj['CONN']
    print_response(conn.volume_create(path, filer_id, categoria_id, max))


@cli.command(name='volume-get')
@click.option('volume_id', '--id', metavar='<volume_id>', type=click.INT,
              required=True)
@click.pass_context
def volume_get(ctx, volume_id):
    conn = ctx.obj['CONN']
    print_response(conn.volume_get(volume_id))


@cli.command(name='volume-delete')
@click.option('volume_id', '--id', metavar='<volume_id>', type=click.INT,
              required=True)
@click.pass_context
def volume_delete(ctx, volume_id):
    conn = ctx.obj['CONN']
    print_response(conn.volume_delete(volume_id))


@cli.command(name='volume-update')
@click.option('volume_id', '--id', metavar='<volume_id>', type=click.INT,
              required=True)
@click.option('--path', metavar='<path>', type=click.STRING, required=True)
@click.option('filer_id', '--filer-id', metavar='<filer_id>', type=click.INT,
              required=True)
@click.option('categoria_id', '--categoria-id', metavar='<categoria_id>',
              type=click.INT, required=True)
@click.option('--max', metavar='<maxsnapshots>', type=click.INT, required=True)
@click.pass_context
def volume_update(ctx, volume_id, path, filer_id, categoria_id, maxsnapshots):
    conn = ctx.obj['CONN']
    print_response(conn.volume_update(volume_id, path, filer_id, categoria_id,
                                      maxsnapshots))


@cli.command(name='volumes-info')
@click.pass_context
def volumes_info(ctx):
    conn = ctx.obj['CONN']
    print_response(conn.volumes_info())


@cli.command(name='volumes-info-update')
@click.pass_context
def volumes_update_info(ctx):
    conn = ctx.obj['CONN']
    print_response(conn.volumes_update_info())


# PERMISSION ------------------------------------------------------------------
@cli.command(name='permission-list')
@click.pass_context
def permission_list(ctx):
    conn = ctx.obj['CONN']
    print_response(conn.permission_list())


@cli.command(name='permission-create')
@click.option('--name', metavar='<permission_name>', type=click.STRING,
              required=True)
@click.pass_context
def permission_create(ctx, name):
    conn = ctx.obj['CONN']
    print_response(conn.permission_create(name))


@cli.command(name='permission-update')
@click.option('permission_id', '--id', metavar='<permission_id>',
              type=click.INT, required=True)
@click.option('--name', metavar='<permission_name>', type=click.STRING,
              required=True)
@click.pass_context
def permission_update(ctx, permission_id, name):
    conn = ctx.obj['CONN']
    print_response(conn.permission_update(permission_id, name))


@cli.command(name='permission-get')
@click.option('permission_id', '--id', metavar='<permission_id>',
              type=click.INT, required=True)
@click.pass_context
def permission_get(ctx, permission_id):
    conn = ctx.obj['CONN']
    print_response(conn.permission_get(permission_id))


@cli.command(name='permission-delete')
@click.option('permission_id', '--id', metavar='<permission_id>',
              type=click.INT, required=True)
@click.pass_context
def permission_delete(ctx, permission_id):
    conn = ctx.obj['CONN']
    print_response(conn.permission_delete(permission_id))


# QUOTA -----------------------------------------------------------------------
@cli.command(name='quota-get')
@click.option('export_id', '--export-id', metavar='<export_id>',
              type=click.INT, required=False)
@click.pass_context
def quota_get(ctx, export_id):
    conn = ctx.obj['CONN']
    print_response(conn.quota_get(export_id))


@cli.command(name='quota-set')
@click.option('export_id', '--export-id', metavar='<export_id>',
              type=click.INT, required=True)
@click.option('--size', metavar='<size in KB>', type=click.INT, required=True)
@click.pass_context
def quota_set(ctx, export_id, size):
    conn = ctx.obj['CONN']
    print_response(conn.export_update(export_id, size))


@cli.command(name='quota-delete')
@click.option('export_id', '--export-id', metavar='<export_id>',
              type=click.INT, required=True)
@click.pass_context
def quota_delete(ctx, export_id):
    conn = ctx.obj['CONN']
    print_response(conn.quota_delete(export_id))


# JOBS / RESTORE --------------------------------------------------------------
@cli.command(name='job-list')
@click.pass_context
def job_list(ctx):
    conn = ctx.obj['CONN']
    print_response(conn.jobs_list())


@cli.command(name='job-get')
@click.option('job_id', '--id', metavar='<job_id>', type=click.STRING,
              required=True)
@click.pass_context
def job_get(ctx, job_id):
    conn = ctx.obj['CONN']
    print_response(conn.jobs_get(job_id))


# SNAPSHOTS -----------------------------------------------------------------
@cli.command(name='snapshot-list')
@click.option('export_id', '--export-id', metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.pass_context
def snapshot_list(ctx, export_id):
    conn = ctx.obj['CONN']
    print_snapshot_list(conn.snapshot_list(export_id))


@cli.command(name='snapshot-create')
@click.option('--export-id', metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.pass_context
def snapshot_create(ctx, export_id):
    conn = ctx.obj['CONN']
    print_response(conn.snapshot_create(export_id))


@cli.command(name='snapshot-get')
@click.option('--export-id', metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.option('--snapshot-id', metavar='<snapshot_id>', type=click.INT,
              required=True)
@click.pass_context
def snapshot_get(ctx, export_id, snapshot_id):
    conn = ctx.obj['CONN']
    print_response(conn.snapshot_get(export_id, snapshot_id))


@cli.command(name='snapshot-delete')
@click.option('export_id', '--export-id', metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.option('snapshot_id', '--snapshot-id', metavar='<snapshot_id>',
              type=click.INT, required=True)
@click.pass_context
def snapshot_delete(ctx, export_id, snapshot_id):
    conn = ctx.obj['CONN']
    print_response(conn.snapshot_delete(export_id, snapshot_id))


@cli.command(name='snapshot-restore')
@click.option('export_id', '--export-id', metavar='<export_id or export_path>',
              type=click.INT, required=True)
@click.option('snapshot_id', '--snapshot-id', metavar='<snapshot_id>',
              type=click.INT, required=True)
@click.pass_context
def snapshot_restore(ctx, export_id, snapshot_id):
    conn = ctx.obj['CONN']
    print_response(conn.snapshot_restore(export_id, snapshot_id))


# REPORT TO GLOBOMAP --------------------------------------------------------------
@cli.command(name='report-get')
@click.pass_context
def report_get(ctx):
    conn = ctx.obj['CONN']
    print(json.dumps(conn.report_get()))


# PRINTS -----------------------------------------------------------------
def print_response(response):

    _, content = response

    if type(content) is list or type(content) is dict:
        print_list(content)
    else:
        print_message(response)


def print_list(content):

    if len(content) == 0:
        click.echo('No results found!')
        return

    if type(content) is dict:
        content = [content]

    fields = sorted(content[0].keys())

    pt = prettytable.PrettyTable([f for f in fields], caching=False)

    for o in content:
        row = []
        for field in fields:
            row.append(o[field])
        pt.add_row(row)

    click.echo(pt.get_string(sortby=fields[0]))


# The dictionary does not have a standard keys! Because it was implemented a
# specific printing method for this case. [There are system snapshots and User
# and only users have id key]
def print_snapshot_list(response):

    _, content = response

    if len(content) == 0:
        click.echo('No results found!')
        return

    if type(content) is dict:
        content = [content]

    keys = sorted(content[0].keys())

    pt = prettytable.PrettyTable([f for f in keys], caching=False)

    for o in content:
        row = []
        for key in keys:
            if 'id' in o:
                row.append(o[key])
            else:
                if key == 'id':
                    row.append('')
                else:
                    row.append(o[key])
        pt.add_row(row)

    click.echo(pt.get_string(sortby=keys[0]))


def print_message(response):
    status_code, content = response

    if status_code in (200, 201):
        content = content or 'Success!'  # For operations return empty response
        click.echo(content)
    else:
        content = content or 'Error!'  # Some operations return empty response
        click.echo('%s (HTTP %d)' % (content, status_code))


if __name__ == '__main__':
    cli(obj={})
